==> auroraview-0.2.4.tar.gz: auroraview-0.2.4/src/ipc/json.rs <==
//! High-performance JSON operations for IPC
//!
//! This module provides orjson-equivalent performance without requiring Python dependencies.
//! All JSON operations are implemented in Rust using simd-json for SIMD acceleration.
//!
//! ## Performance Benefits:
//! - **2-3x faster** than standard serde_json (SIMD acceleration)
//! - **Zero Python dependencies** - no need to install orjson
//! - **Direct PyO3 integration** - optimal Rust ↔ Python conversion
//! - **Memory efficient** - zero-copy parsing where possible
//!
//! ## Implementation:
//! - Uses simd-json for parsing (same as orjson's core)
//! - Direct conversion to Python objects via PyO3
//! - Optimized for IPC message patterns (small to medium JSON)

use pyo3::prelude::*;
use pyo3::types::{PyDict, PyList};
use serde::{Deserialize, Serialize};

// Re-export Value type
pub use serde_json::Value;

/// Parse JSON from a string slice using SIMD acceleration
///
/// This is 2-3x faster than serde_json::from_str() for typical IPC messages.
/// Uses simd-json's SIMD instructions for parallel parsing.
#[inline]
pub fn from_str(s: &str) -> Result<Value, String> {
    // simd-json requires mutable input for zero-copy parsing
    let mut bytes = s.as_bytes().to_vec();

    // Parse with simd-json
    simd_json::serde::from_slice(&mut bytes).map_err(|e| format!("JSON parse error: {}", e))
}

/// Parse JSON from mutable bytes (zero-copy, most efficient)
///
/// This is the fastest parsing method as simd-json can work directly
/// on the mutable buffer without any copying. Use this when you have
/// ownership of the byte buffer.
///
/// # Performance
/// - Zero allocations for parsing
/// - SIMD-accelerated parsing
/// - ~3x faster than serde_json for medium-sized JSON
#[inline]
#[allow(dead_code)]
pub fn from_slice(bytes: &mut [u8]) -> Result<Value, String> {
    // Parse with simd-json
    simd_json::serde::from_slice(bytes).map_err(|e| format!("JSON parse error: {}", e))
}

/// Parse JSON from owned bytes (optimized for IPC)
///
/// This is the recommended method for IPC messages as it:
/// - Takes ownership of the buffer (no copy needed)
/// - Uses SIMD acceleration
/// - Returns a static Value (no lifetime issues)
#[inline]
#[allow(dead_code)]
pub fn from_bytes(mut bytes: Vec<u8>) -> Result<Value, String> {
    // Parse with simd-json
    simd_json::serde::from_slice(&mut bytes).map_err(|e| format!("JSON parse error: {}", e))
}

/// Serialize a value to JSON string
///
/// Uses serde_json for serialization as simd-json's serialization
/// performance is similar and serde_json has better compatibility.
#[inline]
pub fn to_string<T: Serialize>(value: &T) -> Result<String, String> {
    serde_json::to_string(value).map_err(|e| format!("JSON serialize error: {}", e))
}

/// Serialize a value to JSON string with pretty printing
#[inline]
#[allow(dead_code)]
pub fn to_string_pretty<T: Serialize>(value: &T) -> Result<String, String> {
    serde_json::to_string_pretty(value).map_err(|e| format!("JSON serialize error: {}", e))
}

/// Deserialize from JSON value
#[inline]
#[allow(dead_code)]
pub fn from_value<T: for<'de> Deserialize<'de>>(value: Value) -> Result<T, String> {
    serde_json::from_value(value).map_err(|e| format!("JSON deserialize error: {}", e))
}

/// Create a JSON value from a serializable type
#[inline]
#[allow(dead_code)]
pub fn to_value<T: Serialize>(value: &T) -> Result<Value, String> {
    serde_json::to_value(value).map_err(|e| format!("JSON value conversion error: {}", e))
}

/// Convert JSON value to Python object
///
/// This is a critical path for IPC performance, converting Rust JSON
/// to Python objects that can be passed to callbacks.
#[allow(deprecated)]
pub fn json_to_python(py: Python, value: &Value) -> PyResult<PyObject> {
    match value {
        Value::Null => Ok(py.None()),
        Value::Bool(b) => Ok(b.into_py(py)),
        Value::Number(n) => {
            if let Some(i) = n.as_i64() {
                Ok(i.into_py(py))
            } else if let Some(f) = n.as_f64() {
                Ok(f.into_py(py))
            } else {
                Ok(n.to_string().into_py(py))
            }
        }
        Value::String(s) => Ok(s.into_py(py)),
        Value::Array(arr) => {
            let py_list = PyList::empty(py);
            for item in arr {
                let py_item = json_to_python(py, item)?;
                py_list.append(py_item.bind(py))?;
            }
            Ok(py_list.into_py(py))
        }
        Value::Object(obj) => {
            let py_dict = PyDict::new(py);
            for (key, val) in obj {
                let py_val = json_to_python(py, val)?;
                py_dict.set_item(key, py_val)?;
            }
            Ok(py_dict.into_py(py))
        }
    }
}

/// Convert Python object to JSON value
///
/// Supports Python types: str, int, float, bool, None, list, dict (with nesting)
pub fn python_to_json(value: &Bound<'_, PyAny>) -> PyResult<Value> {
    // Try basic types first
    if let Ok(s) = value.extract::<String>() {
        return Ok(Value::String(s));
    }

    if let Ok(i) = value.extract::<i64>() {
        return Ok(Value::Number(i.into()));
    }

    if let Ok(f) = value.extract::<f64>() {
        return Ok(serde_json::json!(f));
    }

    if let Ok(b) = value.extract::<bool>() {
        return Ok(Value::Bool(b));
    }

    // Check for None
    if value.is_none() {
        return Ok(Value::Null);
    }

    // Check for list
    if let Ok(list) = value.downcast::<PyList>() {
        let mut json_array = Vec::new();
        for item in list.iter() {
            json_array.push(python_to_json(&item)?);
        }
        return Ok(Value::Array(json_array));
    }

    // Check for dict
    if let Ok(dict) = value.downcast::<PyDict>() {
        let mut json_obj = serde_json::Map::new();
        for (key, val) in dict.iter() {
            let key_str = key.extract::<String>()?;
            let json_val = python_to_json(&val)?;
            json_obj.insert(key_str, json_val);
        }
        return Ok(Value::Object(json_obj));
    }

    // Unsupported type - convert to string representation
    Ok(Value::String(value.to_string()))
}

#[cfg(test)]
mod tests {
    use super::*;
    use pyo3::types::{PyDict as PyDictType, PyList as PyListType};
    use pyo3::Python;

    #[test]
    fn test_from_str_and_to_string_roundtrip() {
        let s = r#"{"a":1,"b":[1,2,3],"c":null}"#;
        let v = from_str(s).expect("parse ok");
        let out = to_string(&v).expect("serialize ok");
        // serde_json normalizes spacing; reparse and compare
        let v2: Value = serde_json::from_str(&out).unwrap();
        assert_eq!(v, v2);
    }

    #[test]
    fn test_json_to_python_nested_objects() {
        let value = serde_json::json!({
            "s": "x",
            "n": 42,
            "f": std::f64::consts::PI,
            "null": null,
            "arr": [1, "y", null],
            "obj": {"k": "v"}
        });
        Python::with_gil(|py| {
            let obj = json_to_python(py, &value).expect("to py ok");
            let back = python_to_json(obj.bind(py)).expect("roundtrip to json");
            assert_eq!(back["s"], "x");
            assert_eq!(back["n"], 42);
            assert!(back["null"].is_null());
            assert_eq!(back["arr"][1], "y");
        });
    }

    #[test]
    fn test_python_to_json_nested_objects() {
        Python::with_gil(|py| {
            let dict = PyDictType::new(py);
            dict.set_item("s", "x").unwrap();
            dict.set_item("i", 7).unwrap();
            dict.set_item("f", 2.5).unwrap();
            dict.set_item("none", py.None()).unwrap();
            let list = PyListType::empty(py);
            list.append(1).unwrap();
            list.append("y").unwrap();
            list.append(py.None()).unwrap();
            dict.set_item("arr", list).unwrap();

            let v = python_to_json(dict.as_any()).expect("to json ok");
            assert_eq!(v["s"], "x");
            assert_eq!(v["i"], 7);
            assert!(v["none"].is_null());
            assert_eq!(v["arr"][1], "y");
        });
    }

    #[test]
    fn test_from_slice_and_from_bytes_and_pretty() {
        let buf = br#"{"k":1}"#.to_vec();
        let mut slice = buf.clone();
        let v1 = from_slice(&mut slice).expect("slice ok");
        let v2 = from_bytes(buf).expect("bytes ok");
        assert_eq!(v1, v2);
        let pretty = to_string_pretty(&v1).expect("pretty ok");
        assert!(pretty.contains("\n"));
    }

    #[test]
    fn test_value_helpers_and_error() {
        #[derive(serde::Serialize, serde::Deserialize, PartialEq, Debug)]
        struct S {
            a: i32,
        }
        let s = S { a: 5 };
        let val = to_value(&s).expect("to_value ok");
        let back: S = from_value(val).expect("from_value ok");
        assert_eq!(back, s);
        // invalid JSON error path
        assert!(from_str("not json").is_err());
    }

    #[test]
    fn test_from_value_error_wrong_type() {
        let val = serde_json::Value::String("x".to_string());
        let res: Result<i32, _> = from_value(val);
        assert!(res.is_err());
    }

    #[test]
    fn test_from_slice_error() {
        let mut bad = b"{".to_vec(); // invalid JSON
        assert!(from_slice(&mut bad).is_err());
    }
}

==> auroraview-0.2.5.tar.gz: auroraview-0.2.5/src/ipc/json.rs <==
//! High-performance JSON operations for IPC
//!
//! This module provides orjson-equivalent performance without requiring Python dependencies.
//! All JSON operations are implemented in Rust using simd-json for SIMD acceleration.
//!
//! ## Performance Benefits:
//! - **2-3x faster** than standard serde_json (SIMD acceleration)
//! - **Zero Python dependencies** - no need to install orjson
//! - **Direct PyO3 integration** - optimal Rust ↔ Python conversion
//! - **Memory efficient** - zero-copy parsing where possible
//!
//! ## Implementation:
//! - Uses simd-json for parsing (same as orjson's core)
//! - Direct conversion to Python objects via PyO3
//! - Optimized for IPC message patterns (small to medium JSON)

use pyo3::prelude::*;
use pyo3::types::{PyDict, PyList};
use pyo3::{Py, PyAny};
use serde::{Deserialize, Serialize};

// Re-export Value type
pub use serde_json::Value;

/// Parse JSON from a string slice using SIMD acceleration
///
/// This is 2-3x faster than serde_json::from_str() for typical IPC messages.
/// Uses simd-json's SIMD instructions for parallel parsing.
#[inline]
pub fn from_str(s: &str) -> Result<Value, String> {
    // simd-json requires mutable input for zero-copy parsing
    let mut bytes = s.as_bytes().to_vec();

    // Parse with simd-json
    simd_json::serde::from_slice(&mut bytes).map_err(|e| format!("JSON parse error: {}", e))
}

/// Parse JSON from mutable bytes (zero-copy, most efficient)
///
/// This is the fastest parsing method as simd-json can work directly
/// on the mutable buffer without any copying. Use this when you have
/// ownership of the byte buffer.
///
/// # Performance
/// - Zero allocations for parsing
/// - SIMD-accelerated parsing
/// - ~3x faster than serde_json for medium-sized JSON
#[inline]
#[allow(dead_code)]
pub fn from_slice(bytes: &mut [u8]) -> Result<Value, String> {
    // Parse with simd-json
    simd_json::serde::from_slice(bytes).map_err(|e| format!("JSON parse error: {}", e))
}

/// Parse JSON from owned bytes (optimized for IPC)
///
/// This is the recommended method for IPC messages as it:
/// - Takes ownership of the buffer (no copy needed)
/// - Uses SIMD acceleration
/// - Returns a static Value (no lifetime issues)
#[inline]
#[allow(dead_code)]
pub fn from_bytes(mut bytes: Vec<u8>) -> Result<Value, String> {
    // Parse with simd-json
    simd_json::serde::from_slice(&mut bytes).map_err(|e| format!("JSON parse error: {}", e))
}

/// Serialize a value to JSON string
///
/// Uses serde_json for serialization as simd-json's serialization
/// performance is similar and serde_json has better compatibility.
#[inline]
pub fn to_string<T: Serialize>(value: &T) -> Result<String, String> {
    serde_json::to_string(value).map_err(|e| format!("JSON serialize error: {}", e))
}

/// Serialize a value to JSON string with pretty printing
#[inline]
#[allow(dead_code)]
pub fn to_string_pretty<T: Serialize>(value: &T) -> Result<String, String> {
    serde_json::to_string_pretty(value).map_err(|e| format!("JSON serialize error: {}", e))
}

/// Deserialize from JSON value
#[inline]
#[allow(dead_code)]
pub fn from_value<T: for<'de> Deserialize<'de>>(value: Value) -> Result<T, String> {
    serde_json::from_value(value).map_err(|e| format!("JSON deserialize error: {}", e))
}

/// Create a JSON value from a serializable type
#[inline]
#[allow(dead_code)]
pub fn to_value<T: Serialize>(value: &T) -> Result<Value, String> {
    serde_json::to_value(value).map_err(|e| format!("JSON value conversion error: {}", e))
}

/// Convert JSON value to Python object
///
/// This is a critical path for IPC performance, converting Rust JSON
/// to Python objects that can be passed to callbacks.
pub fn json_to_python(py: Python, value: &Value) -> PyResult<Py<PyAny>> {
    match value {
        Value::Null => Ok(py.None()),
        Value::Bool(b) => {
            let obj = b.into_pyobject(py)?;
            Ok(obj.as_any().clone().unbind())
        }
        Value::Number(n) => {
            if let Some(i) = n.as_i64() {
                let obj = i.into_pyobject(py)?;
                Ok(obj.as_any().clone().unbind())
            } else if let Some(f) = n.as_f64() {
                let obj = f.into_pyobject(py)?;
                Ok(obj.as_any().clone().unbind())
            } else {
                let obj = n.to_string().into_pyobject(py)?;
                Ok(obj.as_any().clone().unbind())
            }
        }
        Value::String(s) => {
            let obj = s.into_pyobject(py)?;
            Ok(obj.as_any().clone().unbind())
        }
        Value::Array(arr) => {
            let py_list = PyList::new(py, arr.iter().map(|_| py.None()))?;
            for (idx, item) in arr.iter().enumerate() {
                let py_item = json_to_python(py, item)?;
                py_list.set_item(idx, py_item)?;
            }
            Ok(py_list.into_any().unbind())
        }
        Value::Object(obj) => {
            let py_dict = PyDict::new(py);
            for (key, val) in obj {
                let py_val = json_to_python(py, val)?;
                py_dict.set_item(key, py_val)?;
            }
            Ok(py_dict.into_any().unbind())
        }
    }
}

/// Convert Python object to JSON value
///
/// Supports Python types: str, int, float, bool, None, list, dict (with nesting)
pub fn python_to_json(value: &Bound<'_, PyAny>) -> PyResult<Value> {
    // Try basic types first
    if let Ok(s) = value.extract::<String>() {
        return Ok(Value::String(s));
    }

    if let Ok(i) = value.extract::<i64>() {
        return Ok(Value::Number(i.into()));
    }

    if let Ok(f) = value.extract::<f64>() {
        return Ok(serde_json::json!(f));
    }

    if let Ok(b) = value.extract::<bool>() {
        return Ok(Value::Bool(b));
    }

    // Check for None
    if value.is_none() {
        return Ok(Value::Null);
    }

    // Check for list
    if let Ok(list) = value.cast::<PyList>() {
        let mut json_array = Vec::new();
        for item in list.iter() {
            json_array.push(python_to_json(&item)?);
        }
        return Ok(Value::Array(json_array));
    }

    // Check for dict
    if let Ok(dict) = value.cast::<PyDict>() {
        let mut json_obj = serde_json::Map::new();
        for (key, val) in dict.iter() {
            let key_str = key.extract::<String>()?;
            let json_val = python_to_json(&val)?;
            json_obj.insert(key_str, json_val);
        }
        return Ok(Value::Object(json_obj));
    }

    // Unsupported type - convert to string representation
    Ok(Value::String(value.to_string()))
}

#[cfg(test)]
mod tests {
    use super::*;
    use pyo3::types::{PyDict as PyDictType, PyList as PyListType};
    use pyo3::Python;

    #[test]
    fn test_from_str_and_to_string_roundtrip() {
        let s = r#"{"a":1,"b":[1,2,3],"c":null}"#;
        let v = from_str(s).expect("parse ok");
        let out = to_string(&v).expect("serialize ok");
        // serde_json normalizes spacing; reparse and compare
        let v2: Value = serde_json::from_str(&out).unwrap();
        assert_eq!(v, v2);
    }

    #[test]
    fn test_json_to_python_nested_objects() {
        let value = serde_json::json!({
            "s": "x",
            "n": 42,
            "f": std::f64::consts::PI,
            "null": null,
            "arr": [1, "y", null],
            "obj": {"k": "v"}
        });
        Python::attach(|py| {
            let obj = json_to_python(py, &value).expect("to py ok");
            let back = python_to_json(obj.bind(py)).expect("roundtrip to json");
            assert_eq!(back["s"], "x");
            assert_eq!(back["n"], 42);
            assert!(back["null"].is_null());
            assert_eq!(back["arr"][1], "y");
            Ok::<(), pyo3::PyErr>(())
        })
        .unwrap();
    }

    #[test]
    fn test_python_to_json_nested_objects() {
        Python::attach(|py| {
            let dict = PyDictType::new(py);
            dict.set_item("s", "x").unwrap();
            dict.set_item("i", 7).unwrap();
            dict.set_item("f", 2.5).unwrap();
            dict.set_item("none", py.None()).unwrap();
            let list = PyListType::new(py, vec![py.None()])?;
            list.append(1).unwrap();
            list.append("y").unwrap();
            list.append(py.None()).unwrap();
            dict.set_item("arr", list).unwrap();

            let v = python_to_json(dict.as_any()).expect("to json ok");
            assert_eq!(v["s"], "x");
            assert_eq!(v["i"], 7);
            assert!(v["none"].is_null());
            assert_eq!(v["arr"][1], "y");
            Ok::<(), pyo3::PyErr>(())
        })
        .unwrap();
    }

    #[test]
    fn test_from_slice_and_from_bytes_and_pretty() {
        let buf = br#"{"k":1}"#.to_vec();
        let mut slice = buf.clone();
        let v1 = from_slice(&mut slice).expect("slice ok");
        let v2 = from_bytes(buf).expect("bytes ok");
        assert_eq!(v1, v2);
        let pretty = to_string_pretty(&v1).expect("pretty ok");
        assert!(pretty.contains("\n"));
    }

    #[test]
    fn test_value_helpers_and_error() {
        #[derive(serde::Serialize, serde::Deserialize, PartialEq, Debug)]
        struct S {
            a: i32,
        }
        let s = S { a: 5 };
        let val = to_value(&s).expect("to_value ok");
        let back: S = from_value(val).expect("from_value ok");
        assert_eq!(back, s);
        // invalid JSON error path
        assert!(from_str("not json").is_err());
    }

    #[test]
    fn test_from_value_error_wrong_type() {
        let val = serde_json::Value::String("x".to_string());
        let res: Result<i32, _> = from_value(val);
        assert!(res.is_err());
    }

    #[test]
    fn test_from_slice_error() {
        let mut bad = b"{".to_vec(); // invalid JSON
        assert!(from_slice(&mut bad).is_err());
    }
}
